# A Stochastic Firm Model that Fits Firm Age and Mean Size Data
# Code by Blair Fix

######################################################################
# This code implements a stochastic firm growth model that compares the relation between mean firm size and the fraction of firms under 3.5 years.
# The model assumes scale free growth (Gibrat's law) with a reflective lower bound at a size of 1.
# Growth rates are drawn from a Laplace distribution.
# The time step interval is 0.5 years.
# Firm growth is discrete. After every time iteration, firm sizes are rounded to the nearest ingteger.
# The code implements multiple iterations of the model over different location and scale parameters of the Laplace growth rate distribution.
# Firm age at time t is calculated as the period since the firm's last reflection.


# Packages
################################################
library(doSNOW)
library(foreach)
library(snow)
library(poweRlaw)
library(tcltk)
library(VGAM)


# Set up clusters
###########################################
cl <- makeCluster(4, type="SOCK")
registerDoSNOW(cl)
clusterSetupRNG (cl, type = "RNGstream")

# Variables
#######################################
# t = number of time iterations
t = 200 
# n =  number of firms in the model
n = 1000   
# l =  number of model iterations
l = 960    


# Progress Bar
##########################
start <- proc.time()
pb <- txtProgressBar(max=l, style=3)
progress <- function(n) setTxtProgressBar(pb, n)
opts <- list(progress=progress)
  
  # Generate location and scale parameter vectors
  ##################################################################################3
  # The upper and lower boundary are two seperate sets of vectors for the location and scale parameters of the laplace distribution.
  # The model loops through each vector, picking one pairing of scale and vector parameters. 
  # These vectors are generated by the functions below (which are arbitrary, choosen so the results fit data).
  # When running the code, pick either the upper or lower bound (comment out the other).
  

  # Lower Boundary
  #######################################
  # log.seq = a function to generate log-spaced sequences
  log.seq = function(x,y,z) { exp(seq(log(x), log(y), length.out = z))}
  loc.1 = 1.01
  loc.2 = 0.75
  location.vector = log.seq(loc.1, loc.2, l)
  z = 1:l/l
  scale.vector =  - 10^(-0.83)*(z-1)^10 + 0.19

    # Upper Boundary
  #######################################
#   loc.1 = 1.01
#   loc.2 = 0.9
#   location.vector = log.seq(loc.1, loc.2, l)
#   z = 1:l/l
#   scale.vector = -10^(-0.67)*(1-z)^1.6 + 0.35
  

  
  fit =  foreach (j = 1:l, .options.snow=opts, .combine=rbind, .packages=c('VGAM', 'poweRlaw') ) %dopar%  {
    
    # set the location and scale parameters for this iteration
    location = location.vector[j]
    scale = scale.vector[j]
    

    # Initial conditions
    ##############################################
    # f = a matrix of firm sizes (time in columns, different firms in rows)
	  f = array(0, dim = c(t, n))      
        
    # tenure.matrix =  matrix that will keep track of firm deaths
	  tenure.matrix = array(NA, dim = c(t, n)) 
    
 	  # set initial conditions (firm sizes at time = 1 are set to an arbitrary power law distribution)
	  f[1,] = rpldis(n, 1, 2)                  
    tenure.matrix[1,] = f[1,]
    
    # Loop over time iterations to generate matrix of firm sizes
    ###########################################################
    for(i in 2:t){
      
	  # gr =  random firm growth rates from laplace distribution (fraction form)
    gr = rlaplace(n, location= location, scale=scale) 
    # negative growth rates are rounded to zero  
	  gr[gr <0] = 0                                    
      
	  # new =  size of each firm after growth
    new = round(f[i-1, ]*gr)
    # tenure matrix keeps track of firms hitting 0 (i.e. dying)
    tenure.matrix[i,] = new   
    # firms that `died' are reflected to size 1 
    new[new == 0] = 1           
    f[i,] = new
    }
    
    
    # Firm age calculations
    #########################################################
    # "die" vector  finds the row and column location (within the tenure matrix) of firm deaths (when size = 0)
    die = which(tenure.matrix ==0, arr.ind = T)
    # age.matrix = matrix for calculating firm age
    age.matrix = array(NA, dim = c(100, n))
    # t.vec = a time vector over which we assume the model has achieved equilibrium (an arbitaray period at the end of the model run)
    t.vec = (t-100):t			
    
    #loop through all time in t.vec
    for ( i in t.vec){
      # remove deaths that occur after the period in question (time = i)
      die.sub = die[die[,1] < i,]
      
      # count number of rows in die.sub (add NA's to avoid a NULL length)
      die.sub = rbind(die.sub, c(NA,NA))
      die.length = length(die.sub[,1])-1
      die.sub = na.omit(die.sub)
      
      # Find the location of each firm's most recent death
      # This occurs when first differences of die.sub column 2 do not equal 0
      die.location = die.sub[-die.length,2] - die.sub[-1,2] 
      die.location = which(die.location != 0 )
      # find the time of each firm's most recent death (column 1 of die.sub)
      year.of.death = die.sub[die.location,1]
      # keep track of all firms that have ever died
      firms.that.died = die.sub[die.location,2]
      
      # firm age equals current time (i) - year of death
      age.have.died = i - year.of.death
      # age of firms that have never died = current time (i)
      age.have.not.died = i
      
      # age = a vector of firm ages
      age = 1:n
      # find the column location of firms that have never died
      firms.havent.died = which(! 1:n %in% firms.that.died)
      # set age of firms that have died
      age[firms.that.died] = age.have.died
      # set age of firms that have never died
      age[firms.havent.died] = age.have.not.died
      # paste result into age.matrix at current time
      age.matrix[i-t.vec[1], ] = age
    }
    
    # Find mean firm size and check if it is stable over time
    #####################################################
    # find the mean firm size in every year
    size = rowMeans(f)
    # do a linear regression of size over the period t.vec
    stable = lm(size[t.vec] ~ t.vec)
    # stable = the slope of the regression
    stable = coef(stable)[2]
    # size =  the mean firm size over the period t.vec
    size = mean(size[t.vec])
    
    # Find fraction of firms under 3.5 years old
    #######################################################
    # newbie = all firms under 3.5 years (equivalent to 7 intervals of 0.5 years)
    newbie = which(age.matrix <= 7)
    # newbie = fraction of firms under 3.5 years
    newbie = length(newbie)/length(as.vector(age.matrix))
    
    data.frame(location, scale, stable, size, newbie)
  }
  
  ##############################################################
  # good = data where the firm size distribution is in equilibrium
  # We assume this is when  size vs. time regression has a slope that is negligable (< 0.01)
  good = fit[fit$stable < 0.01,]
  

########################################################################
stopCluster(cl)
end <- proc.time()
time = end - start
close(pb)

plot(good$size, good$newbie, col="red", log="xy")



