# A Management Model
# Code by Blair Fix

###############################################################################################################
# This code generates a power law firm size distribution and then aranges firms into hierarchical levels dictated by the span of control.
# It then calculates the fraction of employees who are managers.
# The model has two parameters, the power law scaling exponent (alpha) and the span of control (s)
# The model uses a Monte Carlo method to  pick s and alpha randomly from a specified range.
# The output is the management fraction and mean firm size for each pair of parameters.
# The loop is implemented in parallel using the foreach package.

# Packages
################################################
library(doSNOW)
library(foreach)
library(snow)
library(poweRlaw)
library(tcltk)

# Clusters (for parallel loop execution)
##################################################
cl <- makeCluster(8, type="SOCK")
registerDoSNOW(cl)
clusterSetupRNG (cl, type = "RNGstream")

# Variables
##################
# s.bottom = lower range for span of control
s.bottom = 2  
# s.top = upper range for span of control
s.top = 5.5   
# l = number of model iterations  
l = 100       
# n.firms = the number of firms in the simultion
n.firms = 10^5

# Generate a vector of  power law exponents (alpha) to input into the model
# The power law exponent ranges from 1.9 to 4, and is distributed according to a power law
# This gives relatively equal spacing of final data when plotted on a log-log scale
#####################################
alpha = rplcon(10^6, 1, 4)    
alpha = alpha[alpha <= 4 & alpha >=1.9 ]
alpha = sample(alpha, l)
alpha = sort(alpha)

# Progress Bar
####################################
startTime <- proc.time()
pb <- txtProgressBar(max=l, style=3)
progress <- function(n) setTxtProgressBar(pb, n)
opts <- list(progress=progress)


# parallel loop over firm size distributions
###############################################################################################
manage.model =  foreach (i = 1:l, .options.snow=opts,  .combine=rbind, .packages=c('poweRlaw') ) %dopar%  {
  
  # generate a random span of control between upper and lower bounds
  s = runif(1, s.bottom, s.top)
  
  # Generate firm size distribution
  ########################################################
  # a = the selected power law exponent
  a = alpha[i]  
  # generate a power law firm size distribution
  n = rpldis(n.firms, 1, a ) 
  # sort it from largest to smallest
  n = sort(n, decreasing = T)

  
  # Generate vectors of span of control, number of levels, employees in base level, 
  #####################################################################################
  # L = vector of the number of hierarchical levels in each firm
  L = log(n*(s-1)+1)/log(s)
  # Round L down to the nearest integer
  L = floor(L)
  # base = a vector of the size of the lowest hierarchical level in each firm
  base = n*(1-1/s)/(1-(1/s)^L)
  # round base up to nearest integer
  base = ceiling(base)
  # exp = an exponent vector used to calculated employment in other hierarchical levels
  exp = 0:19    
  
  # transform vectors into matrices for calculation purposes
  exp.matrix = rbind(exp)[rep(1,n.firms), ]
  s.matrix = cbind(s)[ , rep(1,20)]
  base.firm.matrix = cbind(base)[ , rep(1,20)]
  
  # Hierarchical Employment
  #############################################################################
  # E.distribution = a matrix of  employment in each hierarchical level of each firm (rows = different firms, columns = hierarchical level)
  E.distribution = base.firm.matrix/s.matrix^exp.matrix 
  # Round employment down to nearest integer
  E.distribution = floor(E.distribution)
  # check for overshoot of total employment within each firm 
  overshoot = rowSums(E.distribution)-n  
  # adjust for overshoot by adding/deducting employment from base hierarchical level
  E.distribution[,1] = E.distribution[,1]-overshoot  
  
  # size = mean firm size of the distribution
  size = mean(n) 
  #  management = managers share of employment (emplooyment in hierarchical levels >= 3)
  management = (1-sum(E.distribution[,1:2])/sum(n))*100 #
  
  rm(L, base, base.firm.matrix, exp, exp.matrix, E.distribution, overshoot)
  gc()
  
  cbind.data.frame(size, management, a, s)
}

stopCluster(cl)
endTime <- proc.time()
time = endTime-startTime
close(pb)


# Plots
################################################################
plot(manage.model$size, manage.model$management, log="xy")





